#include "StdAfx.h"
#include "NonModalFileDialog.h"


bool CNonModalFileDialog::registeredDummy = false;

CNonModalFileDialog::CNonModalFileDialog(
      CNonModalFileDialogParent *myParent,
      bool bMultiSelectable,
      LPCTSTR lpszPath,
		bool bOpenFileDialog,
		LPCTSTR lpszDefExt,
		LPCTSTR lpszFileName,
		DWORD dwFlags,
		LPCTSTR lpszFilter,
		CWnd* pParentWnd)
{
	_myParent = myParent;
   _multiSelectable = bMultiSelectable;
   _lpszPath = lpszPath;
	_bOpenFileDialog = bOpenFileDialog;
	_lpszDefExt = lpszDefExt;
	_lpszFileName = lpszFileName;
	_dwFlags = dwFlags;
	_lpszFilter = lpszFilter;
	_pParentWnd = pParentWnd;

   //------------------------
   // Crank it up.
   // Calls InitInstance ()
   //------------------------
   CreateThread ();	
};

CNonModalFileDialog::~CNonModalFileDialog ()
{
   //-----------
   // Clean up
   //-----------
   if (_dlg != NULL)
      delete _dlg;
}

void CNonModalFileDialog::Show (void)
{
	_dlg->SetForegroundWindow ();
}

BOOL CNonModalFileDialog::InitInstance() 
{ 
	WNDCLASS wndclass;

	//----------------------------------------------------
	// First time only,
	// register a dummy window class for later reference.
	//----------------------------------------------------
	if (registeredDummy ==  false)
	{
		::ZeroMemory(&wndclass, sizeof(WNDCLASS));

		wndclass.lpfnWndProc = AfxWndProc;
		wndclass.lpszClassName = "DummyWindowClass";
		wndclass.hInstance = AfxGetInstanceHandle();

		if (!AfxRegisterClass(&wndclass))
		{
			AfxMessageBox("Registration failed");
			return false;   
		}

		registeredDummy = true;
	}
   
   if (_pParentWnd == NULL)
   {
      //-------------------------------------------------------
      // Create a dummy window that never gets
      // displayed or does anything at all.
      // Pass this as the CFileDialog's parent to prevent the
      // taskbar from making an entry for the
      // (otherwise unattached) CFileDialog instance.
      //-------------------------------------------------------
      if (!wind.Create("DummyWindowClass", _T("DummyMainWnd"), 0))
      {
         AfxMessageBox("Create failed");
         return false;   
      }

      _pParentWnd = &wind;
   }      


   //------------------------------------------------------
   // Instantiate a CFileDialog using supplied parameters.
   //------------------------------------------------------
	_dlg = new CFileDialog (_bOpenFileDialog, (LPCSTR)_lpszDefExt, (LPCSTR)_lpszFileName,
		_dwFlags, (LPCSTR)_lpszFilter, _pParentWnd);


   //---------------------------------------------
   // Update the dialog's m_ofn to set things
   // like multi-selectable and the buffer size.
   //---------------------------------------------
   if (_multiSelectable == true)
   {
      _dlg->m_ofn.Flags |= (OFN_ALLOWMULTISELECT);

      //------------------------------------------------
	   // Enlarge the buffer that holds the list of
	   // selected filenames. (<path><file1><file2>...)
	   // Default of 256 is too small.
	   //
	   // You can't just realloc the damn buffer!
	   // I got this solution from codeguru.com
	   //------------------------------------------------
	   int maxBuffer = 2048;
	   CString strFileName;
	   _dlg->m_ofn.lpstrFile = strFileName.GetBuffer(maxBuffer);
      _dlg->m_ofn.nMaxFile = maxBuffer;
   }

   _dlg->m_ofn.lpstrInitialDir = (LPCSTR)_lpszPath;
   strcpy (_dlg->m_ofn.lpstrFile, (LPCSTR)_lpszFileName);



   //---------------------------------------------------------
   // Set the thread's m_pMainWnd.
   // Subsequent calls to dlg->SetWindowForeground will thus 
   // bring the dlg window to the foreground.
   //---------------------------------------------------------
	m_pMainWnd = _dlg;
	
	if (_dlg->DoModal() == IDOK)
	{
      //---------------------------------
      // Acquire the path and filename.
      //---------------------------------
      _lpszPath = _dlg->GetPathName ();
      _lpszFileName = _dlg->GetFileName ();
      if (!_lpszPath.IsEmpty () && !_lpszFileName.IsEmpty ())
      {
         _lpszPath.SetAt (
			   _lpszPath.GetLength() - _lpszFileName.GetLength(),
			   '\0');
      }

		if (_myParent != NULL)
		{
         //--------------------------------------
         // Tell the CNonModalFileDialogParent
         // that the dialog returned IDOK.
         // Pass the dlg back for processing.
         //--------------------------------------
			_myParent->OnFileDialogOK (_dlg);
		}
	}
	else
	{
		if (_myParent != NULL)
		{
         //--------------------------------------
         // Tell the CNonModalFileDialogParent
         // that the dialog returned IDOK.
         //--------------------------------------
			_myParent->OnFileDialogCANCEL ();
		}
	}

   //-----------
   // Clean up
   //-----------
   delete _dlg;
   _dlg = NULL;

	return FALSE;
   //return TRUE;
} 